@@ -39,7 +39,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <X11/keysym.h>
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 #include <X11/extensions/XInput.h>
 #endif
 #include <gconf/gconf.h>
@@ -155,7 +155,7 @@ gsd_mouse_manager_class_init (GsdMouseManagerClass *klass)
 }
 
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static gboolean
 supports_xinput_devices (void)
 {
@@ -226,7 +226,7 @@ configure_button_layout (guchar   *buttons,
         }
 }
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static gboolean
 xinput_device_has_buttons (XDeviceInfo *device_info)
 {
@@ -320,7 +320,7 @@ devicepresence_filter (GdkXEvent *xevent,
         if (xev->type == xi_presence)
         {
             XDevicePresenceNotifyEvent *dpn = (XDevicePresenceNotifyEvent *) xev;
-            if (dpn->devchange == DeviceAdded)
+            if (dpn->devchange == DeviceEnabled)
                 set_mouse_settings ((GsdMouseManager *) data);
         }
         return GDK_FILTER_CONTINUE;
@@ -329,15 +329,24 @@ devicepresence_filter (GdkXEvent *xevent,
 static void
 set_devicepresence_handler (GsdMouseManager *manager)
 {
-        Display *display = gdk_x11_get_default_xdisplay ();
+        Display *display;
         XEventClass class_presence;
         int xi_presence;
 
+        if (!supports_xinput_devices ())
+                return;
+
+        display = gdk_x11_get_default_xdisplay ();
+
+        gdk_error_trap_push ();
         DevicePresence (display, xi_presence, class_presence);
         XSelectExtensionEvent (display,
                                RootWindow (display, DefaultScreen (display)),
                                &class_presence, 1);
-        gdk_window_add_filter (NULL, devicepresence_filter, manager);
+
+        gdk_flush ();
+        if (!gdk_error_trap_pop ())
+                gdk_window_add_filter (NULL, devicepresence_filter, manager);
 }
 #endif
 
@@ -349,7 +358,7 @@ set_left_handed (GsdMouseManager *manager,
         gsize buttons_capacity = 16;
         gint n_buttons, i;
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         if (supports_xinput_devices ()) {
                 set_xinput_devices_left_handed (left_handed);
         }
@@ -714,7 +723,7 @@ gsd_mouse_manager_idle_cb (GsdMouseManager *manager)
                                           GCONF_MOUSE_A11Y_DIR,
                                           (GConfClientNotifyFunc) mouse_callback);
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         set_devicepresence_handler (manager);
 #endif
         set_mouse_settings (manager);
@@ -769,7 +778,7 @@ gsd_mouse_manager_stop (GsdMouseManager *manager)
 
         set_locate_pointer (manager, FALSE);
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         gdk_window_remove_filter (NULL, devicepresence_filter, manager);
 #endif
 }
